package com.persagy.dmp.fss.ioc.tool;

import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.nio.charset.Charset;

public class HttpTools {

    public static String postJson(String url,String json) throws Exception{
        String result = null;

        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();

            HttpPost httpPost = new HttpPost(url);
            httpPost.addHeader("Content-type","application/json; charset=utf-8");
            httpPost.setHeader("Accept", "application/json");
            httpPost.setEntity(new StringEntity(json, Charset.forName("UTF-8")));

            response = httpClient.execute(httpPost);

            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != 200 && statusCode != 201) {
                httpPost.abort();
                throw new RuntimeException("HttpClient,error status code :" + statusCode);
            }

            entity = response.getEntity();


            if (entity != null){
                result = EntityUtils.toString(entity, "utf-8");
            }
        } finally {
            if(entity!=null){
                EntityUtils.consume(entity);
            }
            if(response!=null){
                response.close();
            }
        }


        return result;
    }

    public static String uploadFile(byte[] body, String filename, String userId, String authorization, String url) throws Exception {
        String result = null;

        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();
            MultipartEntityBuilder meb = MultipartEntityBuilder.create();
            meb.addBinaryBody("file", body, ContentType.DEFAULT_BINARY, filename);
            meb.addPart("userId",new StringBody(userId,ContentType.create("text/plain", Charset.forName("UTF-8"))));
            HttpEntity reqEntity = meb.build();
            HttpPost httpPost = new HttpPost(url);
            httpPost.setEntity(reqEntity);
            httpPost.addHeader("Authorization", authorization);
            response = httpClient.execute(httpPost);
            entity = response.getEntity();

            if (entity != null){
                result = EntityUtils.toString(entity, "utf-8");
            }
        } finally {
            if(entity!=null){
                EntityUtils.consume(entity);
            }
            if(response!=null){
                response.close();
            }
        }


        return result;
    }

    public static byte[] downloadFile(String url) throws Exception {
        byte[] content = null;
        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        try {
            CloseableHttpClient client = HttpClients.createDefault();
            HttpGet httpget = new HttpGet(url);
            response = client.execute(httpget);
            entity = response.getEntity();
            InputStream inputStream = entity.getContent();
            content = readInputStream(inputStream);
        } finally {
            if(entity!=null){
                EntityUtils.consume(entity);
            }
            if(response!=null){
                response.close();
            }
        }
        return content;
    }

    public static byte[] readInputStream(InputStream inputStream) throws Exception {
        ByteArrayOutputStream bos = null;
        try {
            byte[] buffer = new byte[1024];
            int len = 0;
            bos = new ByteArrayOutputStream();
            while((len = inputStream.read(buffer)) != -1) {
                bos.write(buffer, 0, len);
            }
        } finally {
            if(bos!=null){
                bos.flush();
                bos.close();
            }
        }
        return bos.toByteArray();
    }

    public static String delete(String url, String authorization) throws Exception {
        String result = null;

        CloseableHttpResponse response = null;
        HttpEntity entity = null;
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpPost httpPost = new HttpPost(url);
            httpPost.addHeader("Authorization", authorization);
            response = httpClient.execute(httpPost);
            entity = response.getEntity();

            if (entity != null){
                result = EntityUtils.toString(entity, "utf-8");
            }
        } finally {
            if(entity!=null){
                EntityUtils.consume(entity);
            }
            if(response!=null){
                response.close();
            }
        }


        return result;
    }
}
