package com.persagy.dmp.fss.common.controller;

import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.fss.FSSApp;
import com.persagy.dmp.fss.common.service.FileService;
import com.persagy.dmp.fss.common.service.TokenService;
import com.persagy.dmp.fss.file.entity.FileResource;
import com.persagy.dmp.fss.file.service.FileResourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.net.URLEncoder;

@RestController
@RequestMapping("/file")
public class FileController {

    @Autowired
    private TokenService tokenService;

    @RequestMapping(value = "/upload", consumes = {"multipart/form-data"})
    public MapResponse upload(@RequestParam(value = "fileName") String fileName,
                              @RequestParam(value = "appId") String appId,
                              @RequestParam(value = "userId") String userId,
                              @RequestParam(value = "groupCode") String groupCode,
                              @RequestParam(value = "projectId", required = false, defaultValue = "unique") String projectId,
                              @RequestParam(value = "file") MultipartFile file
                              ) throws Exception {

        MapResponse mapResponse = new MapResponse();
        FileService fileService = null;
        try {
            fileService = SpringUtils.getBean(FileService.class);
            String id = fileService.upload(fileName, appId, userId, groupCode, projectId, file);
            mapResponse.add("id", id);
        } catch (Exception e) {
            e.printStackTrace();
            mapResponse.setResult(BaseResponse.FAIL);
            mapResponse.setMessage(e.getMessage());
            mapResponse.add("error", FSSApp.getExceptionStackTrace(e));
        }

        return mapResponse;
    }

    @RequestMapping(value = "/download")
    public ResponseEntity<byte[]> download(@RequestParam(value = "appId") String appId, @RequestParam(value = "resourceId") String resourceId) throws Exception {

        FileService fileService = SpringUtils.getBean(FileService.class);
        FileResourceService fileResourceService = SpringUtils.getBean(FileResourceService.class);
        FileResource fileResource = fileResourceService.getFileResource(resourceId);
        byte[] body = fileService.download(fileResource);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        headers.setContentDispositionFormData("attachment", URLEncoder.encode(fileResource.getFileName(), "UTF-8"));

        ResponseEntity<byte[]> responseEntity = new ResponseEntity<byte[]>(body, headers, HttpStatus.CREATED);
        return responseEntity;
    }

    @RequestMapping(value = "/delete")
    public MapResponse delete(@RequestParam(value = "appId") String appId, @RequestParam(value = "resourceId") String resouceId) throws Exception {
        MapResponse mapResponse = new MapResponse();
        FileService fileService = null;
        try {
            fileService = SpringUtils.getBean(FileService.class);
            fileService.delete(resouceId);
        } catch (Exception e) {
            e.printStackTrace();
            mapResponse.setResult(BaseResponse.FAIL);
            mapResponse.setMessage(e.getMessage());
            mapResponse.add("error", FSSApp.getExceptionStackTrace(e));
        }

        return mapResponse;
    }
}
