package com.persagy.dmp.fss.common.controller;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.fss.FSSApp;
import com.persagy.dmp.fss.common.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/token")
public class TokenController {

    @Autowired
    private TokenService tokenService;

    @RequestMapping(value = "/get", consumes = {"application/json"})
    public MapResponse getToken(@RequestBody ObjectNode json) throws Exception {
        String accessKey = json.get("accessKey").asText();
        String secretKey = json.get("secretKey").asText();
        String token = null;
        MapResponse mapResponse = new MapResponse();
        try {
            token = tokenService.generateToken(accessKey, secretKey);
            mapResponse.add("token", token);
        } catch (Exception e) {
            e.printStackTrace();
            mapResponse.setResult(BaseResponse.FAIL);
            mapResponse.setMessage(e.getMessage());
            mapResponse.add("error", FSSApp.getExceptionStackTrace(e));
        }

        return mapResponse;
    }

    @RequestMapping(value = "/check", consumes = {"application/json"})
    public MapResponse checkToken(@RequestBody ObjectNode json) throws Exception {
        String appId = json.get("appId").asText();
        String token = json.get("token").asText();
        Boolean check = null;
        MapResponse mapResponse = new MapResponse();
        try {
            tokenService.checkToken(appId, token);
        } catch (Exception e) {
            e.printStackTrace();
            mapResponse.setResult(BaseResponse.FAIL);
            mapResponse.setMessage(e.getMessage());
            mapResponse.add("error", FSSApp.getExceptionStackTrace(e));
        }

        return mapResponse;
    }
}
