package com.persagy.dmp.fss.common.interceptor;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.persagy.common.web.BaseResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.common.SpringUtils;
import com.persagy.dmp.fss.FSSApp;
import com.persagy.dmp.fss.common.service.TokenService;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

public class AuthorizationInterceptor implements HandlerInterceptor {

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        String token = request.getHeader("token");
        String appId = request.getParameter("appId");
        try {
            if(StringUtils.isBlank(token)){
                token = request.getParameter("auth");
                if(StringUtils.isBlank(token)){
                    throw new Exception("参数错误！1");
                }

            }
            if(StringUtils.isBlank(appId)){
                throw new Exception("参数错误！2");
            }
            TokenService tokenService = SpringUtils.getBean(TokenService.class);
            tokenService.checkToken(appId, token);
        } catch (Exception e) {
            e.printStackTrace();
            MapResponse mapResponse = new MapResponse();
            mapResponse.setResult(BaseResponse.FAIL);
            mapResponse.setMessage(e.getMessage());
            mapResponse.add("error", FSSApp.getExceptionStackTrace(e));
            ObjectMapper objectMapper = SpringUtils.getBean(ObjectMapper.class);
            returnJson(response, objectMapper.writeValueAsString(mapResponse));
            return false;
        }
        return true;
    }

    private void returnJson(HttpServletResponse response, String json) throws Exception{
        PrintWriter writer = null;
        response.setCharacterEncoding("UTF-8");
        response.setContentType("text/html; charset=utf-8");
        try {
            writer = response.getWriter();
            writer.print(json);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (writer != null)
                writer.close();
        }
    }

}
