package com.persagy.dmp.fss.common.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.fss.file.entity.FileResource;
import com.persagy.dmp.fss.file.repository.FileResourceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Optional;
import java.util.UUID;

public abstract class BaseFileService implements FileService {

    @Autowired
    private FileResourceRepository fileResourceRepository;
    @Autowired
    protected ObjectMapper objectMapper;

    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    @Override
    public String upload(String fileName, String appId, String userId, String groupCode, String projectId, MultipartFile file) throws Exception {
        ObjectNode extension = store(fileName, appId, groupCode, projectId, file);
        String resourceId = record(file.getSize(), fileName, appId, userId,groupCode, projectId, extension);
        return resourceId;
    }

    private String record(Long size, String fileName,String appId, String userId, String groupCode, String projectId, ObjectNode extension) throws Exception {
        FileResource entity = new FileResource();
        entity.setId(UUID.randomUUID().toString());
        entity.setAppId(appId);
        entity.setGroupCode(groupCode);
        entity.setProjectId(projectId);
        entity.setFileName(fileName);
        if(extension==null){
            extension = objectMapper.createObjectNode();
        }
        LocalDateTime localDateTime = LocalDateTime.now();
        String format = localDateTime.format(formatter);
        extension.put("date", format);
        extension.put("type", this.getType());
        extension.put("size", size);
        entity.setExtension(extension);
        fileResourceRepository.save(entity);
        return entity.getId();
    }

    protected abstract ObjectNode store(String fileName, String appId, String groupCode, String projectId, MultipartFile file) throws Exception;

    @Override
    public void delete(String resourceId) throws Exception {
        FileResource fileResource = deleteRecord(resourceId);
        if(fileResource!=null) {
            clear(fileResource);
        }
    }

    private FileResource deleteRecord(String resourceId) throws Exception {
        Optional<FileResource> of = fileResourceRepository.findById(resourceId);
        if(!of.isPresent()){
            return null;
        }
        fileResourceRepository.deleteById(resourceId);
        return of.get();
    }

    protected abstract void clear(FileResource fileResource) throws Exception;

    protected abstract String getType();
}
