package com.persagy.dmp.fss.common.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.persagy.dmp.fss.common.service.TokenService;
import com.persagy.dmp.fss.common.tool.AESTools;
import com.persagy.dmp.fss.file.entity.FileAuthorization;
import com.persagy.dmp.fss.file.entity.QFileAuthorization;
import com.persagy.dmp.fss.file.repository.FileAuthorizationRepository;
import com.querydsl.jpa.impl.JPAQuery;
import com.querydsl.jpa.impl.JPAQueryFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Service
public class TokenServiceImpl implements TokenService {

    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private  ObjectMapper objectMapper;

    @Autowired
    private FileAuthorizationRepository fileAuthorizationRepository;

    @Autowired
    private JPAQueryFactory queryFactory;

    @Override
    public String generateToken(String accessKey, String secretKey) throws Exception {
        QFileAuthorization q = QFileAuthorization.fileAuthorization;
        JPAQuery<FileAuthorization> query = queryFactory.selectFrom(q).where(q.accessKey.eq(accessKey).and(q.secretKey.eq(secretKey)));
        List<FileAuthorization> list = query.fetch();
        if(list.size()==0){
            throw new Exception("访问验证失败！0");
        }

        LocalDateTime localDateTime = LocalDateTime.now().plusMinutes(1);
        String format = localDateTime.format(formatter);

        Map tokenMap = new LinkedHashMap<>();
        tokenMap.put("expireTime", format);
        tokenMap.put("accessKey", accessKey);
        tokenMap.put("secretKey", secretKey);

        String plainText = objectMapper.writeValueAsString(tokenMap);
        String token = AESTools.encrypt(plainText);
        return token;
    }

    @Override
    public void checkToken(String appId, String token) throws Exception {
        String plainText = null;
        try {
            plainText = AESTools.decrypt(token);
        } catch (Exception e) {
            throw new Exception("访问验证失败！0", e);
        }
        Map tokenMap = objectMapper.readValue(plainText, Map.class);
        String accessKey = (String) tokenMap.get("accessKey");
        String secretKey = (String) tokenMap.get("secretKey");
        String expireTime = (String) tokenMap.get("expireTime");

        Optional<FileAuthorization> of = fileAuthorizationRepository.findById(appId);
        if(!of.isPresent()){
            throw new Exception("访问验证失败！1");
        }

        FileAuthorization fileAuthorization = of.get();
        if(!fileAuthorization.getAccessKey().equals(accessKey) || !fileAuthorization.getSecretKey().equals(secretKey)){
            throw new Exception("访问验证失败！2");
        }

        LocalDateTime localDateTime = LocalDateTime.parse(expireTime, formatter);
        if(localDateTime.compareTo(LocalDateTime.now()) < 0 ){
            throw new Exception("访问验证失败！3");
        }

    }

}
